<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Schplurtz le Déboulonné <Schplurtz@laposte.net>
 * @author Hérisson grognon <dodoperso@laposte.net>
 * @author Nicolas Friedli <nicolas@theologique.ch>
 * @author tuxun <tuxuntrash@gmail.com>
 * @author olinuxx/trebmuh <trebmuh@tuxfamily.org>
 */
$lang['export_pdf_button']     = 'Exporter en PDF';
$lang['needtitle']             = 'Veuillez choisir un titre.';
$lang['needns']                = 'Veuillez choisir une catégorie existante.';
$lang['notexist']              = 'La page demandée n\'existe pas.';
$lang['empty']                 = 'Vous n\'avez encore sélectionné aucune page.';
$lang['tocheader']             = 'Table des matières';
$lang['export_ns']             = 'Exporter la catégorie  « %s » dans le fichier %s.pdf.';
$lang['missingbookcreator']    = 'Le greffon Bookcreator est désactivé ou n\'est pas installé.';
