<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Eduardo Mozart de Oliveira <eduardomozart182@gmail.com>
 * @author Paulo Schopf <pschopf@gmail.com>
 * @author Rodrigo Pimenta <rodrigo.pimenta@gmail.com>
 */
$lang['export_pdf_button']     = 'Exportar para PDF';
$lang['needtitle']             = 'Favor fornecer um título.';
$lang['needns']                = 'Favor fornecer um namespace existente.';
$lang['notexist']              = 'A página solicitada não existe.';
$lang['empty']                 = 'Você ainda não selecionou páginas.';
$lang['tocheader']             = 'Tabela de Conteúdos';
$lang['export_ns']             = 'Exportar o namespace "%s:" para arquivo %s.pdf';
$lang['missingbookcreator']    = 'O Plugin Bookcreator não está instalado ou está desabilitado';
