<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Domingo Redal <docxml@gmail.com>
 */
$lang['export_pdf_button']     = 'Exportar a PDF';
$lang['needtitle']             = 'Por favor proporcione un título.';
$lang['needns']                = 'Por favor proporcione un nombre de espacio existente.';
$lang['notexist']              = 'La página solicitada no existe.';
$lang['empty']                 = 'No tiene páginas seleccionadas aún.';
$lang['tocheader']             = 'Tabla de Contenidos';
$lang['export_ns']             = 'Exportar el espacio de nombres "%s:" al fichero%s.pdf';
