<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Nima Hamidian <nima.hamidian@gmail.com>
 * @author Sam01 <m.sajad079@gmail.com>
 */
$lang['export_pdf_button']     = 'تبدیل به پی‌دی‌اف';
$lang['needtitle']             = 'لطفا یک عنوان ارائه دهید.';
$lang['needns']                = 'لطفا یک فضای‌نام موجود ارائه دهید.';
$lang['notexist']              = 'صفحه‌ی درخواست‌شده موجود نمی‌باشد.';
$lang['empty']                 = 'شما صفحه‌های انتخاب شده‌ای ندارید.';
$lang['tocheader']             = 'جدول محتوا';
$lang['export_ns']             = 'تبدیل فضای‌نام "%s:" به فایل %s.pdf';
$lang['missingbookcreator']    = 'پلاگین کتابساز نصب‌ نشده یا غیرفعال است';
