<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Petr Kajzar <petr.kajzar@centrum.cz>
 * @author Robert Surý <rsurycz@seznam.cz>
 * @author Kamil Nešetřil <kamil.nesetril@volny.cz>
 * @author Jaroslav Lichtblau <jlichtblau@seznam.cz>
 */
$lang['export_pdf_button']     = 'Uložit do PDF';
$lang['needtitle']             = 'Zadejte prosím název.';
$lang['needns']                = 'Zadejte prosím existující jmenný prostor.';
$lang['notexist']              = 'Požadovaná stránka neexistuje';
$lang['empty']                 = 'Zatím nejsou vybrány žádné stránky.';
$lang['tocheader']             = 'Obsah';
$lang['export_ns']             = 'Exportovat jmenný prostor "%s:" do souboru %s.pdf';
$lang['missingbookcreator']    = 'Zásuvný modul Bookcreator není nainstalován nebo je zakázán';
