<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author piano <linyixuan2019@hotmail.com>
 * @author nero <dreamfox225@hotmail.com>
 * @author oott123 <ip.192.168.1.1@qq.com>
 * @author maddie <2934784245@qq.com>
 */
$lang['export_pdf_button']     = '导出 PDF';
$lang['needtitle']             = '需要指定标题';
$lang['needns']                = '请提供现有命名空间';
$lang['notexist']              = '请求的页面不存在。';
$lang['empty']                 = '您还没有选择的页面。';
$lang['tocheader']             = '目录';
$lang['export_ns']             = '将命名空间"%s:"转至%s.pdf文件';
$lang['missingbookcreator']    = 'Bookcreator 插件未安装或被禁用';
