<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author David Surroca <david.tb303@gmail.com>
 */
$lang['export_pdf_button']     = 'Exporta a PDF';
$lang['needtitle']             = 'Proporcioneu un títol.';
$lang['needns']                = 'Proporcioneu un espai de noms existent.';
$lang['empty']                 = 'Encara no teniu cap pàgina seleccionada.';
$lang['tocheader']             = 'Índex';
