<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Марко М. Костић <marko.m.kostic@gmail.com>
 */
$lang['export_pdf_button']     = 'Извези у ПДФ';
$lang['needtitle']             = 'Наведите наслов.';
$lang['needns']                = 'Наведите већ постојећи именски простор.';
$lang['empty']                 = 'Немате изабраних страница још.';
$lang['tocheader']             = 'Садржај';
$lang['export_ns']             = 'Извези именски простор „%s:“ у датотеку %s.pdf';
