/**
 * These are the styles loaded by default
 *
 * Do not edit this file. Instead add your own styles in
 * lib/plugins/dw2pdf/conf/style.local.css or create your own template
 * to make sure they are not overwritten by updates
 */

body {
    font-family: sans-serif;
}

a {
    text-decoration: none;
}

table.inline {
    border: 1px solid #808080;
    border-collapse: collapse;
}
table.inline td, th {
    border: 1px solid #808080;
}
th {
    text-align: left;
}

pre {
    background-color: #eeeeee;
    color: #000000;
    padding: 0.3em;
}

em.u {
    font-style: normal;
    text-decoration: underline;
}
em em.u {
    font-style: italic;
}

img.medialeft {
    float: left;
}

img.mediaright {
    float: right;
}

blockquote {
    border-left: 3px solid #cccccc;
    margin: 0;
    padding: 5px 0 5px 15px;
}

.rightalign {
    text-align: right;
}

.centeralign {
    text-align: center;
}

.leftalign {
    text-align: left;
}

img.icon.smiley {
    height: 1.2em;
}
