<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Daniel Dias Rodrigues <danieldiasr@gmail.com>
 * @author Schopf <pschopf@gmail.com>
 */
$lang['export_pdf_button']     = 'Exportar para PDF';
$lang['needtitle']             = 'Digite um título.';
$lang['needns']                = 'Digite um domínio existente.';
$lang['notexist']              = 'A página solicitada não existe.';
$lang['empty']                 = 'Você não selecionou nenhuma página.';
$lang['tocheader']             = 'Índice';
$lang['export_ns']             = 'Exportar domínio "%s:" para o arquivo %s.pdf';
$lang['missingbookcreator']    = 'O plugin Bookcreator não está instalado ou está desativado';
