<?php

/**
 * @license    GPL 2 (https://www.gnu.org/licenses/gpl.html)
 *
 * @author Wizzard <wizzardsk@gmail.com>
 */
$lang['export_pdf_button']     = 'Uložiť do PDF';
$lang['needtitle']             = 'Prosím, zadajte názov.';
$lang['needns']                = 'Prosím, zadajte existujúci menný priestor.';
$lang['notexist']              = 'Požadovaná stránka neexistuje.';
$lang['empty']                 = 'Ešte ste nevybrali žiadne stránky.';
$lang['tocheader']             = 'Obsah';
$lang['export_ns']             = 'Uložiť menný priestor "%s:" do súboru %s.pdf';
$lang['missingbookcreator']    = 'Plugin Bookcreator nie je nainštalovaný alebo je zakázaný';
