<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Markus <mosesmami0@gmail.com>
 * @author Sebastian Engel <mail@engel-s.de>
 * @author Juergen-aus-Koeln <h-j-schuemmer@web.de>
 * @author F. Mueller-Donath <j.felix@mueller-donath.de>
 */
$lang['export_pdf_button']     = 'PDF exportieren';
$lang['needtitle']             = 'Bitte Titel angeben!';
$lang['needns']                = 'Bitte geben Sie einen vorhandenen Namensraum an.';
$lang['notexist']              = 'Die angeforderte Seite existiert nicht.';
$lang['empty']                 = 'Sie haben noch keine Seiten gewählt.';
$lang['tocheader']             = 'Inhaltsverzeichnis';
$lang['export_ns']             = 'Exportiere Namensraum "%s:" in Datei %s.pdf';
$lang['missingbookcreator']    = 'Das Plugin "Bookcreator" ist entweder nicht installiert oder deaktiviert.';
