<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Aleksandr Selivanov <alexgearbox@yandex.ru>
 * @author FOTONTV <obraztsov568@gmail.com>
 * @author Yuriy Skalko <yuriy.skalko@gmail.com>
 * @author Vasilyy Balyasnyy <v.balyasnyy@gmail.com>
 * @author RainbowSpike <1@2.ru>
 */
$lang['export_pdf_button']     = 'Экспорт в PDF';
$lang['needtitle']             = 'Пожалуйста, укажите заголовок.';
$lang['needns']                = 'Пожалуйста, укажите существующее пространство имён.';
$lang['notexist']              = 'Запрошенная страница не существует.';
$lang['empty']                 = 'У вас ещё нет выделенных страниц.';
$lang['tocheader']             = 'Содержание';
$lang['export_ns']             = 'Экспортировать пространство имён «%s:» в файл %s.pdf';
$lang['missingbookcreator']    = 'Плагин Bookcreator не установлен или отключён';
