<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Otto Riehl <post@otto-riehl.de>
 */
$lang['export_pdf_button']     = 'Експорт до PDF';
$lang['needtitle']             = 'Будь ласка, вкажіть назву.';
$lang['needns']                = 'Будь ласка, вкажіть існуючий простір імен.';
$lang['notexist']              = 'Запитувана сторінка не існує.';
$lang['empty']                 = 'Ви ще не вибрали жодної сторінки.';
$lang['tocheader']             = 'Зміст';
$lang['export_ns']             = 'Експортувати простір імен "%s:" до файлу %s.pdf';
$lang['missingbookcreator']    = 'Плагін Bookcreator не встановлено або вимкнено';
