<?php

$lang['export_pdf_button'] = "Export to PDF";
$lang['needtitle']         = "Please provide a title.";
$lang['needns']            = "Please provide an existing namespace.";
$lang['notexist']          = "Requested page doesn't exist.";
$lang['empty']             = "You don't have pages selected yet.";
$lang['tocheader']         = "Table of Contents";
$lang['export_ns']         = 'Export namespace "%s:" to file %s.pdf';
$lang['forbidden']         = "You have no read access to the selected pages.";
$lang['exportfailed']      = "The PDF could not be created.";
$lang['missingbookcreator'] = 'The Bookcreator Plugin is not installed or is disabled';
