<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Tor Härnqvist <tor@harnqvist.se>
 */
$lang['export_pdf_button']     = 'Exportera till PDF';
$lang['needtitle']             = 'Vad god fyll i rubrik';
$lang['needns']                = 'Var god fyll i en existerande namnrymd';
$lang['notexist']              = 'Begärd sida existerar inte.';
$lang['empty']                 = 'Du har inte valt några sidor än.';
$lang['tocheader']             = 'Innehållsföreteckning';
$lang['export_ns']             = 'Exportera namnrymd "%s:" till filen %s.pdf';
$lang['missingbookcreator']    = 'Bokskaparpluginet (Bookcreator) är inte installerat eller är avaktiverat';
