<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author qezwan <qezwan@gmail.com>
 */
$lang['export_pdf_button']     = 'هەناردەی بکە بۆ PDF';
$lang['needtitle']             = 'تکایە ناونیشانێک دابین بکە.';
$lang['needns']                = 'تکایە بۆشایی ناوی بەردەست دابین بکە.';
$lang['notexist']              = 'لاپەڕەی داواکراو بوونی نییە.';
$lang['empty']                 = 'تۆ هێشتا پەڕەت دەست نیشان نەکراوە.';
$lang['tocheader']             = 'خشتەی ناوەڕۆکەکان';
$lang['export_ns']             = 'ناردنی بۆشایی ناوی ناوی "%s:" بۆ فایلی %s.pdf';
