<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Minh <phandinhminh@protonmail.ch>
 * @author Thien Hau <thienhau.9a14@gmail.com>
 */
$lang['export_pdf_button']     = 'Xuất ra PDF';
$lang['needtitle']             = 'Vui lòng cung cấp tiêu đề.';
$lang['needns']                = 'Vui lòng cung cấp một không gian tên hiện có.';
$lang['notexist']              = 'Trang được yêu cầu không tồn tại.';
$lang['empty']                 = 'Bạn chưa chọn trang nào.';
$lang['tocheader']             = 'Mục lục';
$lang['export_ns']             = 'Xuất không gian tên "%s:" ra tập tin %s.pdf';
$lang['missingbookcreator']    = 'Tiện ích plugin Bookcreator chưa được cài đặt hoặc bị tắt đi';
