<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author Jacob Palm <mail@jacobpalm.dk>
 * @author Søren Birk <soer9648@eucl.dk>
 */
$lang['export_pdf_button']     = 'Eksporter til PDF';
$lang['needtitle']             = 'Angiv venligst en titel.';
$lang['needns']                = 'Angiv venligst et eksisterende navnerum.';
$lang['notexist']              = 'Den forespurgte side findes ikke.';
$lang['empty']                 = 'Du har endnu ikke valgt sider.';
$lang['tocheader']             = 'Indholdsfortegnelse';
$lang['export_ns']             = 'Eksporter navnerummet "%s:" ti filen %s.pdf';
