<?php

/**
 * This file is part of FPDI
 *
 * @package   setasign\Fpdi
 * @copyright Copyright (c) 2026 Setasign GmbH & Co. KG (https://www.setasign.com)
 * @license   http://opensource.org/licenses/mit-license The MIT License
 */

namespace setasign\Fpdi;

/**
 * Class TcpdfFpdi
 *
 * This class let you import pages of existing PDF documents into a reusable structure for TCPDF.
 *
 * @deprecated Class was moved to \setasign\Fpdi\Tcpdf\Fpdi
 */
class TcpdfFpdi extends \setasign\Fpdi\Tcpdf\Fpdi
{
    // this class is moved to \setasign\Fpdi\Tcpdf\Fpdi
}
