<?php

$conf['pagesize']         = 'A4';
$conf['orientation']      = 'portrait';
$conf['font-size']        = 11;
$conf['doublesided']      = 1;
$conf['toc']              = 0;
$conf['toclevels']        = '';
$conf['headernumber']     = 0;
$conf['maxbookmarks']     = 5;
$conf['watermark']        = '';
$conf['template']         = 'default';
$conf['output']           = 'browser';
$conf['usecache']         = 1;
$conf['usestyles']        = 'wrap,';
$conf['qrcodescale']       = '1';
$conf['showexportbutton'] = 1;
