<?php return array(
    'root' => array(
        'name' => 'splitbrain/dw2pdf',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => '3ddddb3b2485c18a6aa6809af789c7abb6977134',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        'mpdf/mpdf' => array(
            'pretty_version' => 'v8.3.1',
            'version' => '8.3.1.0',
            'reference' => '2a454ec334109911fdb323a284c19dbf3f049810',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/mpdf',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mpdf/psr-http-message-shim' => array(
            'pretty_version' => 'v2.0.1',
            'version' => '2.0.1.0',
            'reference' => 'f25a0153d645e234f9db42e5433b16d9b113920f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/psr-http-message-shim',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mpdf/psr-log-aware-trait' => array(
            'pretty_version' => 'v2.0.0',
            'version' => '2.0.0.0',
            'reference' => '7a077416e8f39eb626dee4246e0af99dd9ace275',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/psr-log-aware-trait',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'mpdf/qrcode' => array(
            'pretty_version' => 'v1.2.2',
            'version' => '1.2.2.0',
            'reference' => 'd4fa19117a7241c30ac84902b6236a02c7a3f268',
            'type' => 'library',
            'install_path' => __DIR__ . '/../mpdf/qrcode',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'myclabs/deep-copy' => array(
            'pretty_version' => '1.13.4',
            'version' => '1.13.4.0',
            'reference' => '07d290f0c47959fd5eed98c95ee5602db07e0b6a',
            'type' => 'library',
            'install_path' => __DIR__ . '/../myclabs/deep-copy',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '*',
            ),
        ),
        'psr/http-message' => array(
            'pretty_version' => '2.0',
            'version' => '2.0.0.0',
            'reference' => '402d35bcb92c70c026d1a6a9883f06b2ead23d71',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'psr/log' => array(
            'pretty_version' => '1.1.4',
            'version' => '1.1.4.0',
            'reference' => 'd49695b909c3b7628b6289db5479a1c204601f11',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/log',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'setasign/fpdi' => array(
            'pretty_version' => 'v2.6.8',
            'version' => '2.6.8.0',
            'reference' => '881945be29a4996ad3d008eb18ddc01fa3df890c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../setasign/fpdi',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'splitbrain/dw2pdf' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => '3ddddb3b2485c18a6aa6809af789c7abb6977134',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
