<?php

/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 *
 * @author weiqi <yuweiqi_001@126.com>
 * @author lioujheyu <lioujheyu@gmail.com>
 */
$lang['export_pdf_button']     = '輸出PDF檔案';
$lang['needtitle']             = '請提供一個抬頭名稱';
$lang['needns']                = '請提供一個以存在的命名空間';
$lang['notexist']              = '您所請求的頁面不存在。';
$lang['empty']                 = '你尚未選擇頁面。';
$lang['tocheader']             = '目錄';
$lang['export_ns']             = '將命名空間「%s:」匯出至檔案 %s.pdf';
$lang['missingbookcreator']    = 'Bookcreator外掛程式未安裝或已停用';
