<?php

namespace Mpdf\Log;

class Context
{

	const STATISTICS = 'statistics';

	const PDFA_PDFX = 'pdfa_pdfx';

	const UTF8 = 'utf8';

	const REMOTE_CONTENT = 'remote_content';

	const IMAGES = 'images';

	const CSS_SIZE_CONVERSION = 'css_size_conversion';

	const HTML_MARKUP = 'html_markup';

}
